@@ -5,12 +5,12 @@
 class CColor {
   public:
     CColor();
-    CColor(float, float, float, float);
+    CColor(float r, float g, float b, float a);
     CColor(uint64_t);
 
     float    r = 0, g = 0, b = 0, a = 1.f;
 
-    uint64_t getAsHex();
+    uint32_t getAsHex();
 
     CColor   operator-(const CColor& c2) const {
         return CColor(r - c2.r, g - c2.g, b - c2.b, a - c2.a);
@@ -27,4 +27,8 @@ class CColor {
     bool operator==(const CColor& c2) const {
         return r == c2.r && g == c2.g && b == c2.b && a == c2.a;
     }
+
+    CColor stripA() const {
+        return {r, g, b, 1};
+    }
 };
